<?php
/*
Plugin Name: A-Select Authentication
Version: 2.0
Plugin URI: http://non-gnu.uvt.nl/
Description: Authenticate users using A-Select. User info retrieved from LDAP.
Author: Wessel Dankers
Author URI: mailto:wsl@uvt.nl
*/

require_once 'aselect/client.php';
require_once ABSPATH.WPINC.'/registration-functions.php';

function generic_setcookie($name, $value = null) {
	$cookiepath = COOKIEPATH;
	$sitecookiepath = SITECOOKIEPATH;
	$secure = $_SERVER['HTTPS'] == 'on';

	if(is_null($value)) {
		/* expiry date in the past(1970-01-02Z) */
		$expire = 86400;
		$value = '';
	} else {
		/* expire at end of browser session */
		$expire = 0;
	}

	setcookie($name, $value, $expire, $cookiepath, COOKIE_DOMAIN, $secure);

	if($cookiepath != $sitecookiepath)
		setcookie($name, $value, $expire, $sitecookiepath, COOKIE_DOMAIN, $secure);
}

function ldapuserinfo($uid) {
	if(!eregi('^[a-z][a-z0-9_]*$', $uid))
		return null;

	$server = get_option('aselect_ldap_server');
	$base = get_option('aselect_ldap_base');
	$filter = get_option('aselect_ldap_filter');

	$attrs = array('cn', 'givenName', 'sn', 'mail');

	$ldap = ldap_connect($server);

	if(!$ldap)
		die("Can't contact LDAP-server");

	if(empty($filter)) {
		$filter = "(uid=$uid)";
	} else {
		if(!ereg('^\(.*\)$', $filter))
			die("Malformed LDAP filter, contact the site administrator");
		$filter = '(&'.$filter."(uid=$uid))";
	}

	$search = ldap_search($ldap, $base, $filter, $attrs, 0, 2);

	if(!$search)
		die("Can't search LDAP-server");

	$entry = ldap_first_entry($ldap, $search);
	if(!$entry)
		return null;

	if(ldap_next_entry($ldap, $entry))
		die("Too many results");

	$all = ldap_get_attributes($ldap, $entry);

	$ret = array();

	foreach($attrs as $attr) {
		$values = ldap_get_values($ldap, $entry, $attr);
		$ret[$attr] = $values[0];
	}

	return $ret;
}

function aselect_login() {
	$aselectinfo = aselect_verify_ticket($_COOKIE['aselect']);
	if(!is_array($aselectinfo))
		return null;
	$uid = $aselectinfo['uid'];

	if(!function_exists('get_userdatabylogin'))
		die("Could not load user data");

	$user = get_userdatabylogin($uid);
	if(!$user) {
		$auto_create = get_option('aselect_auto_create');
		if($auto_create != 'on')
			die("User ID '$uid' doesn't exist yet; contact the admin to have it created.");
		$ldap = ldapuserinfo($uid);
		if(is_null($ldap))
			die("User ID '$uid' can't be created automatically; contact the admin.");
		$user_id = wp_create_user($uid, $ldap['cn'], $ldap['mail']);
		update_usermeta($user_id, 'first_name', $ldap['givenName']);
		update_usermeta($user_id, 'last_name', $ldap['sn']);
		$user = get_userdatabylogin($uid);
		if(!$user)
			die("Can't create account for '$uid': contact the admin.");
	}

	if($user->user_login != $uid)
		die("Can't locate account for '$uid': contact the admin.");

	return $uid;
}

function aselect_force() {
	$uid = aselect_login();
	if(!is_null($uid))
		return $uid;

	$cred = $_REQUEST['aselect_credentials'];
	if(!empty($cred)) {
		$rid = $_REQUEST['rid'];
		if(!empty($rid)) {
			$ticket = aselect_verify_credentials($rid, $cred);
			if(!is_null($ticket)) {
				generic_setcookie('aselect', $ticket);
				wp_redirect($_SERVER['REQUEST_URI']);
				exit();
			}
		}
	}

	$sso = aselect_authenticate('wordpress');
	if(empty($sso))
		die("A-Select problem: agent won't give me an SSO redirection URL");
	wp_redirect($sso);
	exit();
}

if(!class_exists('AselectAuthenticationPlugin')) {
	class AselectAuthenticationPlugin {
		function AselectAuthenticationPlugin() {
			if(isset($_GET['activate']) and $_GET['activate'] == 'true') {
				add_action('init', array(&$this, 'init'));
			}
			add_action('admin_menu', array(&$this, 'admin_menu'));
			add_action('wp_authenticate', array(&$this, 'authenticate'), 10, 2);
			add_action('wp_logout', array(&$this, 'logout'));
			add_action('lost_password', array(&$this, 'disable_function'));
			add_action('retrieve_password', array(&$this, 'disable_function'));
			add_action('password_reset', array(&$this, 'disable_function'));
			add_action('check_passwords', array(&$this, 'check_passwords'), 10, 3);
			add_filter('show_password_fields', array(&$this, 'show_password_fields'));
		}


		/*************************************************************
		 * Plugin hooks
		 *************************************************************/

		/*
		 * Add options for this plugin to the database.
		 */
		function init() {
			if(current_user_can('manage_options')) {
				add_option('aselect_logout_uri', get_option('home'), 'The URI to which the user is redirected when she chooses "Logout".');
				add_option('aselect_allow_anon', 'no', 'Allow anonymous users.');
				add_option('aselect_auto_create', 'no', 'Automatically create accounts.');
				add_option('aselect_ldap_server', 'ldaps://ldap.example.com', 'The URL to the LDAP server');
				add_option('aselect_ldap_base', 'dc=example,dc=com', 'The base DN of the LDAP server');
				add_option('aselect_ldap_filter', '(organizationalStatus=staff)', 'Additional filter for new users');
			}
		}

		/*
		 * Add an options pane for this plugin.
		 */
		function admin_menu() {
			if(function_exists('add_options_page')) {
				add_options_page('A-Select Authentication', 'A-Select', 9, 'wp-content/plugins/aselect-authentication.php', array(&$this, 'display_options_page'));
			}
		}

		function authenticate($username, $password) {
			$user = aselect_force();

			$username = $user;
			$password = true;
		}

		/*
		 * Logout the user by redirecting them to the logout URI.
		 */
		function logout() {
			$location = get_option('http_authentication_logout_uri');
			if(empty($location))
				$location = get_option('home');
			generic_setcookie('aselect');
			header('Location: ' . $location);
			exit();
		}

		/*
		 * Generate a password for the user. This plugin does not
		 * require the user to enter this value, but we want to set it
		 * to something nonobvious.
		 */
		function check_passwords($username, $password1, $password2) {
			$password1 = $password2 = md5(uniqid(rand(), true));
		}

		/*
		 * Used to disable certain display elements, e.g. password
		 * fields on profile screen.
		 */
		function show_password_fields($show_password_fields) {
			return false;
		}

		/*
		 * Used to disable certain login functions, e.g. retrieving a
		 * user's password.
		 */
		function disable_function() {
			die('Disabled');
		}


		/*************************************************************
		 * Functions
		 *************************************************************/

		/*
		 * Display the options for this plugin.
		 */
		function display_options_page() {
			$logout_uri = get_option('http_authentication_logout_uri');
			$auto_create = get_option('aselect_auto_create');
			$allow_anon = get_option('aselect_allow_anon');
			$mail_domain = get_option('aselect_mail_domain');
			$ldap_server = get_option('aselect_ldap_server');
			$ldap_base = get_option('aselect_ldap_base');
			$ldap_filter = get_option('aselect_ldap_filter');
?>
<div class="wrap">
  <h2>A-Select Authentication Options</h2>
  <form action="options.php" method="post">
    <input type="hidden" name="action" value="update" />
    <input type="hidden" name="page_options" value="aselect_logout_uri,aselect_auto_create,aselect_allow_anon,aselect_ldap_server,aselect_ldap_base,aselect_ldap_filter" />
    <?php if(function_exists('wp_nonce_field')): wp_nonce_field('update-options'); endif; ?>

    <fieldset class="options">
      <table class="editform optiontable">
        <tr valign="top">
          <th scope="row"><label for="aselect_logout_uri">Logout URI</label></th>
          <td>
            <input type="text" name="aselect_logout_uri" id="aselect_logout_uri" value="<?php echo htmlspecialchars($logout_uri); ?>" size="50" /><br />
            Default is <code><?php echo htmlspecialchars(get_settings('home')); ?></code>.
          </td>
        </tr>
        <tr valign="top">
          <th scope="row"><label for="aselect_auto_create">Automatically create accounts</label></th>
          <td>
            <input type="checkbox" name="aselect_auto_create" id="aselect_auto_create" <?php echo checked($auto_create, 'on'); ?>" value="on" />
          </td>
        </tr>
        <tr valign="top">
          <th scope="row"><label for="aselect_allow_anon">Allow anonymous users</label></th>
          <td>
            <input type="checkbox" name="aselect_allow_anon" id="aselect_allow_anon" <?php echo checked($allow_anon, 'on'); ?>" value="on" />
          </td>
        </tr>
        <tr valign="top">
          <th scope="row"><label for="aselect_ldap_server">The URL to the LDAP server</label></th>
          <td>
            <input type="text" name="aselect_ldap_server" id="aselect_ldap_server" value="<?php echo htmlspecialchars($ldap_server); ?>" size="50" />
          </td>
        </tr>
        <tr valign="top">
          <th scope="row"><label for="aselect_ldap_base">The base DN of the LDAP server</label></th>
          <td>
            <input type="text" name="aselect_ldap_base" id="aselect_ldap_base" value="<?php echo htmlspecialchars($ldap_base); ?>" size="50" />
          </td>
        </tr>
        <tr valign="top">
          <th scope="row"><label for="aselect_ldap_filter">Additional filter for new users</label></th>
          <td>
            <input type="text" name="aselect_ldap_filter" id="aselect_ldap_filter" value="<?php echo htmlspecialchars($ldap_filter); ?>" size="50" />
          </td>
        </tr>
      </table>
    </fieldset>
    <p class="submit">
      <input type="submit" name="Submit" value="Update Options &raquo;" />
    </p>
  </form>
</div>
<?php
		}
	}
}

$using_cookie = false;
$aselect_plugin = new AselectAuthenticationPlugin();

if(!function_exists('get_currentuserinfo')):
function get_currentuserinfo() {
	global $current_user;

	if(defined('XMLRPC_REQUEST') && XMLRPC_REQUEST)
		return false;

	if(!empty($current_user))
		return true;

	$allow_anon = get_option('aselect_allow_anon');
	if($allow_anon == 'on')
		$user = aselect_login();
	else
		$user = aselect_force();

	if(!$user) {
		wp_set_current_user(0);
		return false;
	}

	wp_set_current_user(0, $user);
}
endif;

if(!function_exists('auth_redirect')):
function auth_redirect() {
    aselect_force();
}
endif;

if(!function_exists('wp_get_cookie_login')):
function wp_get_cookie_login() {
	$allow_anon = get_option('aselect_allow_anon');
	if($allow_anon == 'on')
		$user = aselect_login();
	else
		$user = aselect_force();

    if(!$user)
        return false;

    return array('login' => $user,  'password' => true);
}
endif;

if(!function_exists('wp_new_user_notification')):
function wp_new_user_notification($user_id, $plaintext_pass = '') {
    $user = new WP_User($user_id);

    $user_login = stripslashes($user->user_login);
    $user_email = stripslashes($user->user_email);

    $message  = sprintf(__('New user registration on your blog %s:'), get_settings('blogname')) . "\r\n\r\n";
    $message .= sprintf(__('Username: %s'), $user_login) . "\r\n\r\n";
    $message .= sprintf(__('E-mail: %s'), $user_email) . "\r\n";

    @wp_mail(get_settings('admin_email'), sprintf(__('[%s] New User Registration'), get_settings('blogname')), $message);
}
endif;

?>
